@@ -21,7 +21,7 @@ from unicorn import (
 )
 from unicorn.unicorn import UC_HOOK_CODE_TYPE
 
-from . import const, hooks
+from . import const
 from .arch import arch_arm, arch_arm64
 from .exceptions import EmulatorCrashedException, SymbolMissingException
 from .log import get_logger
@@ -66,69 +66,65 @@ class Infernum:
         self.trace_inst = trace_inst
         self.trace_symbol_calls = trace_symbol_calls
 
-        self.endian: Literal["little", "big"] = "little"
+        self.endian = const.LITTLE_ENDIAN
+
+        self.modules: List[Module] = []
 
         self.uc = self._create_uc()
         self.cs = self._create_cs()
 
         self.memory_manager = MemoryManager(uc=self.uc, heap_address=const.HEAP_ADDRESS)
 
-        self._module_address = const.MODULE_ADDRESS
-        self._trap_address = const.TRAP_ADDRESS
-
-        self.modules: List[Module] = []
-        self._symbol_hooks: Dict[str, UC_HOOK_CODE_TYPE] = {}
-
+        self._traps: List[int] = []
         self._init_trap_memory()
+
+        self._symbol_hooks: Dict[str, UC_HOOK_CODE_TYPE] = {}
         self._init_symbol_hooks()
 
         self._setup_emulator()
 
+    @property
+    def _module_address_offset(self) -> int:
+        """Get offset address of modules."""
+        if self.modules:
+            return aligned(self.modules[-1].base + self.modules[-1].size, 1024 * 1024)
+        return const.MODULE_ADDRESS
+
+    @property
+    def _trap_address_offset(self) -> int:
+        """Get offset address of traps."""
+        if self._traps:
+            return const.TRAP_ADDRESS + len(self._traps) * 4
+        return const.TRAP_ADDRESS
+
     def _create_uc(self) -> Uc:
         """Create Unicorn instance."""
         arch = UC_ARCH_ARM if self.arch == arch_arm else UC_ARCH_ARM64
         mode = UC_MODE_THUMB if self.arch == arch_arm and self.on_thumb else UC_MODE_ARM
+
         return Uc(arch, mode)
 
     def _create_cs(self) -> Cs:
         """Create Capstone instance."""
         arch = CS_ARCH_ARM if self.arch == arch_arm else CS_ARCH_ARM64
         mode = CS_MODE_THUMB if self.arch == arch_arm and self.on_thumb else CS_MODE_ARM
+
         return Cs(arch, mode)
 
     def _init_symbol_hooks(self):
         """Initialize default symbol hooks."""
-        _hooks = {
-            "__ctype_get_mb_cur_max": hooks.hook_ctype_get_mb_cur_max,
-            "malloc": hooks.hook_malloc,
-            "getcwd": hooks.hook_getcwd,
-            "getpid": hooks.hook_getpid,
-            "gettid": hooks.hook_gettid,
-            "free": hooks.hook_free,
-        }
-
-        if self.arch == arch_arm:
-            _hooks.update({})
-
-        elif self.arch == arch_arm64:
-            _hooks.update(
-                {
-                    "arc4random": hooks.hook_arc4random,
-                    "clock_nanosleep": hooks.hook_clock_nanosleep,
-                    "nanosleep": hooks.hook_nanosleep,
-                }
-            )
-
-        self._symbol_hooks.update(_hooks)
+        self._symbol_hooks.update(self.arch.symbol_hooks)
 
     def _init_trap_memory(self):
-        """Initialize trap memory."""
+        """Initialize trap area memory."""
         self.uc.mem_map(const.TRAP_ADDRESS, const.TRAP_SIZE)
 
     def _setup_stack(self):
         """Setup stack."""
         stack_addr = const.STACK_ADDRESS + const.STACK_SIZE // 2
+
         self.uc.mem_map(const.STACK_ADDRESS, const.STACK_SIZE)
+
         self.uc.reg_write(self.arch.reg_sp, stack_addr)
         self.uc.reg_write(self.arch.reg_fp, stack_addr)
 
@@ -165,7 +161,9 @@ class Infernum:
 
         self.uc.mem_map(const.TEMP_ADDRESS, aligned(len(inst_code), 1024))
         self.uc.mem_write(addr, inst_code)
+
         self.uc.emu_start(addr | 1, addr + len(inst_code))
+
         self.uc.mem_unmap(const.TEMP_ADDRESS, aligned(len(inst_code), 1024))
 
     def _setup_emulator(self):
@@ -189,7 +187,7 @@ class Infernum:
         self.uc.reg_write(self.arch.reg_pc, address)
 
         try:
-            self.logger.info(f"Start emulate at {self.get_location(address)}.")
+            self.logger.info(f"Start emulate at {self.locate_address(address)}.")
             self.uc.emu_start(address, stop_addr)
 
         except UcError as e:
@@ -200,7 +198,7 @@ class Infernum:
             # Check whether the last instruction is svc.
             if inst.mnemonic == "svc":
                 raise EmulatorCrashedException(
-                    f"Unhandled system call at {self.get_location(address)}"
+                    f"Unhandled system call at {self.locate_address(address)}"
                 )
 
             raise EmulatorCrashedException("Unknown reason") from e
@@ -209,7 +207,7 @@ class Infernum:
         """Trace instruction."""
         for inst in self.cs.disasm_lite(uc.mem_read(address, size), 0):
             self.logger.info(
-                f"Trace at {self.get_location(address)}: {inst[-2]} {inst[-1]}."
+                f"Trace at {self.locate_address(address)}: {inst[-2]} {inst[-1]}."
             )
 
     def _trace_symbol_call_callback(self, *args):
@@ -220,7 +218,7 @@ class Infernum:
 
         self.logger.info(
             f"Symbol '{symbol.name}' called"
-            + (f" from {self.get_location(ret_addr)}." if ret_addr else ".")
+            + (f" from {self.locate_address(ret_addr)}." if ret_addr else ".")
         )
 
     @staticmethod
@@ -247,20 +245,20 @@ class Infernum:
 
         raise SymbolMissingException(f"{symbol_name} not found")
 
-    def locate_module(self, address: int) -> Optional[Module]:
-        """Locate the module according to the address."""
+    def locate_address(self, address: int) -> Location:
+        """Locate which module this address is in."""
+        located_module = None
+
         for module in self.modules:
             if module.base <= address < module.base + module.size:
-                return module
-        return None
+                located_module = module
+                break
 
-    def get_location(self, address: int) -> Location:
-        """Get location for the address."""
-        return Location(address=address, module=self.locate_module(address))
+        return Location(address=address, module=located_module)
 
     def backtrace(self) -> List[Location]:
-        """Back trace the call stack."""
-        addr_list = [self.uc.reg_read(self.arch.reg_lr)]
+        """Backtrace the call stack."""
+        call_stack = [self.uc.reg_read(self.arch.reg_lr)]
         fp = self.uc.reg_read(self.arch.reg_fp)
 
         while True:
@@ -268,9 +266,16 @@ class Infernum:
             fp = self.read_int(fp)
             if not fp or not lr:
                 break
-            addr_list.append(lr)
+            call_stack.append(lr)
 
-        return [self.get_location(addr - 4) for addr in addr_list if addr]
+        return [self.locate_address(addr - 4) for addr in call_stack if addr]
+
+    def return_call(self, retval: Optional[int] = None):
+        """Return current function call."""
+        if retval is not None:
+            self.set_retval(retval)
+
+        self.uc.reg_write(self.arch.reg_pc, self.uc.reg_read(self.arch.reg_lr))
 
     def add_hook(
         self,
@@ -278,8 +283,8 @@ class Infernum:
         callback: UC_HOOK_CODE_TYPE,
         user_data: Optional[dict] = None,
         silenced: Optional[bool] = False,
-    ):
-        """Add hook to emulator.
+    ) -> int:
+        """Add hook to the emulator.
 
         Args:
             symbol_or_addr: The name of symbol or address to hook. If ``str``,
@@ -300,18 +305,18 @@ class Infernum:
 
             if not silenced:
                 self.logger.info(
-                    f"Hook symbol '{symbol.name}' at {self.get_location(hook_addr)}."
+                    f"Hook symbol '{symbol.name}' at {self.locate_address(hook_addr)}."
                 )
 
         else:
             hook_addr = symbol_or_addr
             if not silenced:
-                self.logger.info(f"Hook address at {self.get_location(hook_addr)}.")
+                self.logger.info(f"Hook address at {self.locate_address(hook_addr)}.")
 
         if user_data is None:
             user_data = {}
 
-        self.uc.hook_add(
+        return self.uc.hook_add(
             UC_HOOK_CODE,
             callback,
             begin=hook_addr,
@@ -319,10 +324,78 @@ class Infernum:
             user_data={"emulator": self, **user_data},
         )
 
-    def _relocate_address(self, relocation: Relocation, segment: DynamicSegment):
+    def del_hook(self, handle: int):
+        """Delete hook."""
+        self.uc.hook_del(handle)
+
+    def _map_segments(self, elffile: ELFFile) -> Tuple[int, int]:
+        """Map segment of type ``LOAD`` into memory."""
+        base = self._module_address_offset
+        boundary = 0
+
+        for segment in elffile.iter_segments(type="PT_LOAD"):
+            seg_addr = base + segment.header.p_vaddr
+
+            # Because of alignment, the actual mapped address range will be larger.
+            map_addr = aligned(seg_addr, 1024) - (1024 if seg_addr % 1024 else 0)
+            map_size = aligned(seg_addr - map_addr + segment.header.p_memsz, 1024)
+
+            self.uc.mem_map(map_addr, map_size)
+            self.uc.mem_write(seg_addr, segment.data())
+
+            boundary = max(boundary, map_addr + map_size)
+
+        return base, boundary - base
+
+    def _get_symbols(self, elffile: ELFFile) -> List[Symbol]:
+        """Get all symbols in the module."""
+        symbols = []
+
+        for segment in elffile.iter_segments(type="PT_DYNAMIC"):
+            for symbol in segment.iter_symbols():
+                if (
+                    symbol.entry["st_info"]["bind"] in ("STB_GLOBAL", "STB_WEAK")
+                    and symbol.entry["st_shndx"] != "SHN_UNDEF"
+                ):
+                    symbols.append(
+                        Symbol(
+                            address=self._module_address_offset
+                            + symbol.entry["st_value"],
+                            name=symbol.name,
+                            type=symbol.entry["st_info"]["type"],
+                        )
+                    )
+
+        return symbols
+
+    def _exec_init_array(self, elffile: ELFFile, module: Module):
+        """Find and execute initialization functions in section ``.init_array``."""
+        init_array_addr = None
+        init_array_size = None
+
+        for segment in elffile.iter_segments(type="PT_DYNAMIC"):
+            for tag in segment.iter_tags():
+                if tag.entry.d_tag == "DT_INIT_ARRAY":
+                    init_array_addr = tag.entry.d_val
+                elif tag.entry.d_tag == "DT_INIT_ARRAYSZ":
+                    init_array_size = tag.entry.d_val
+
+        if init_array_addr is not None and init_array_size is not None:
+            self.logger.info("Execute initialize functions.")
+
+            for offset in range(0, init_array_size, self.arch.reg_size // 8):
+                init_func_addr = self.read_int(module.base + init_array_addr + offset)
+
+                if init_func_addr:
+                    # Execute the initialization functions.
+                    self._start_emulate(init_func_addr)
+
+    def _relocate_address(
+        self, module_base: int, relocation: Relocation, segment: DynamicSegment
+    ):
         """Relocate address in the relocation table."""
         reloc_type = relocation["r_info_type"]
-        reloc_offset = self._module_address + relocation["r_offset"]
+        reloc_offset = module_base + relocation["r_offset"]
         reloc_addr = None
 
         if reloc_type in (
@@ -342,25 +415,32 @@ class Infernum:
             except SymbolMissingException:
                 # If the symbol is missing, let it jump to the trap address to
                 # raise an exception with useful information.
-                self.write_int(reloc_offset, self._trap_address)
                 self.add_hook(
-                    self._trap_address,
+                    self._trap_address_offset,
                     self._missing_symbol_required_callback,
                     user_data={"symbol_name": symbol_name},
                     silenced=True,
                 )
-                self._trap_address += 4
+                self.write_int(reloc_offset, self._trap_address_offset)
+                self._traps.append(self._trap_address_offset)
 
         elif reloc_type in (
             ENUM_RELOC_TYPE_ARM["R_ARM_RELATIVE"],
             ENUM_RELOC_TYPE_AARCH64["R_AARCH64_RELATIVE"],
         ):
             if relocation.is_RELA():
-                reloc_addr = self._module_address + relocation["r_addend"]
+                reloc_addr = module_base + relocation["r_addend"]
 
         if reloc_addr:
             self.write_int(reloc_offset, reloc_addr)
 
+    def _process_relocation(self, elffile: ELFFile, module_base: int):
+        """Process relocation tables."""
+        for segment in elffile.iter_segments(type="PT_DYNAMIC"):
+            for relocation_table in segment.get_relocation_tables().values():
+                for relocation in relocation_table.iter_relocations():
+                    self._relocate_address(module_base, relocation, segment)
+
     def load_module(
         self,
         module_file: str,
@@ -378,98 +458,49 @@ class Infernum:
                 disassemble results.
             trace_symbol_calls: If ``True``, trace symbol calls in this module.
         """
-        module_name = os.path.basename(module_file)
-        elffile = ELFFile.load_from_path(module_file)
+        with ELFFile.load_from_path(module_file) as elffile:
+            module_name = os.path.basename(module_file)
+            self.logger.info(f"Load module '{module_name}'.")
 
-        self.logger.info(f"Load module '{module_name}'.")
+            # Map segments into memory.
+            base, size = self._map_segments(elffile)
+            symbols = self._get_symbols(elffile)
 
-        # The memory range in which the module is loaded.
-        low_addr = self._module_address
-        high_addr = 0
+            module = Module(base=base, size=size, name=module_name, symbols=symbols)
+            self.modules.append(module)
 
-        module = Module(base=low_addr, size=0, name=module_name, symbols=[])
-        self.modules.append(module)
-
-        init_array_addr = None
-        init_array_size = None
-
-        # Load segment of type `LOAD` into memory.
-        for segment in elffile.iter_segments(type="PT_LOAD"):
-            seg_addr = low_addr + segment.header.p_vaddr
-
-            # Because of alignment, the actual mapped address range will be larger.
-            map_addr = aligned(seg_addr, 1024) - (1024 if seg_addr % 1024 else 0)
-            map_size = aligned(seg_addr - map_addr + segment.header.p_memsz, 1024)
-
-            high_addr = max(high_addr, map_addr + map_size)
-
-            self.uc.mem_map(map_addr, map_size)
-            self.uc.mem_write(seg_addr, segment.data())
-
-        module.size = high_addr - low_addr
-
-        for segment in elffile.iter_segments(type="PT_DYNAMIC"):
-            for symbol in segment.iter_symbols():
-                if (
-                    symbol.entry["st_info"]["bind"] in ("STB_GLOBAL", "STB_WEAK")
-                    and symbol.entry["st_shndx"] != "SHN_UNDEF"
-                ):
-                    module.symbols.append(
-                        Symbol(
-                            address=module.base + symbol.entry["st_value"],
-                            name=symbol.name,
+            for symbol in symbols:
+                if symbol.type == "STT_FUNC":
+                    # Trace symbol call
+                    if trace_symbol_calls or self.trace_symbol_calls:
+                        self.add_hook(
+                            symbol.name,
+                            self._trace_symbol_call_callback,
+                            user_data={"symbol": symbol},
+                            silenced=True,
                         )
-                    )
 
                     # Hook symbol
-                    if symbol.entry["st_info"]["type"] == "STT_FUNC":
-                        # Trace
-                        if trace_symbol_calls or self.trace_symbol_calls:
-                            self.add_hook(
-                                symbol.name,
-                                self._trace_symbol_call_callback,
-                                user_data={"symbol": symbol},
-                                silenced=True,
-                            )
-
-                        if self._symbol_hooks.get(symbol.name):
-                            self.add_hook(symbol.name, self._symbol_hooks[symbol.name])
-
-            for tag in segment.iter_tags():
-                if tag.entry.d_tag == "DT_NEEDED":
-                    pass
-                elif tag.entry.d_tag == "DT_INIT_ARRAY":
-                    init_array_addr = tag.entry.d_val
-                elif tag.entry.d_tag == "DT_INIT_ARRAYSZ":
-                    init_array_size = tag.entry.d_val
+                    if self._symbol_hooks.get(symbol.name):
+                        self.add_hook(symbol.name, self._symbol_hooks[symbol.name])
 
             # Process address relocation.
-            for relocation_table in segment.get_relocation_tables().values():
-                for relocation in relocation_table.iter_relocations():
-                    self._relocate_address(relocation, segment)
+            self._process_relocation(elffile, base)
 
-        if trace_inst or self.trace_inst:
-            # Trace instructions
-            self.uc.hook_add(
-                UC_HOOK_CODE, self._trace_inst_callback, begin=low_addr, end=high_addr
-            )
+            if trace_inst or self.trace_inst:
+                # Trace instructions
+                self.uc.hook_add(
+                    UC_HOOK_CODE,
+                    self._trace_inst_callback,
+                    begin=module.base,
+                    end=module.base + module.size,
+                )
 
-        # If the section `.init_array` exists.
-        if (
-            exec_init_array
-            and init_array_addr is not None
-            and init_array_size is not None
-        ):
-            self.logger.info("Execute initialize functions.")
-            for offset in range(0, init_array_size, self.arch.reg_size // 8):
-                init_func_addr = self.read_int(low_addr + init_array_addr + offset)
+            if exec_init_array:
+                # Execute initialization functions.
+                self._exec_init_array(elffile, module)
 
-                if init_func_addr:
-                    # Execute the initialization functions.
-                    self._start_emulate(init_func_addr)
-
-        self._module_address = aligned(high_addr, 1024 * 1024)
-        return module
+            return module
 
     def _get_argument_holder(self, index: int) -> Tuple[bool, int]:
         """Get the register or address where the specified argument is stored.
@@ -497,16 +528,20 @@ class Infernum:
     def get_argument(self, index: int) -> int:
         """Get argument with the specified index."""
         is_reg, reg_or_addr = self._get_argument_holder(index)
+
         if is_reg:
             return self.uc.reg_read(reg_or_addr)
-        return self.read_int(reg_or_addr)
+        else:
+            return self.read_int(reg_or_addr)
 
     def set_argument(self, index: int, value: int):
         """Set argument with the specified index."""
         is_reg, reg_or_addr = self._get_argument_holder(index)
+
         if is_reg:
-            return self.uc.reg_write(reg_or_addr, value)
-        self.write_int(reg_or_addr, value)
+            self.uc.reg_write(reg_or_addr, value)
+        else:
+            self.write_int(reg_or_addr, value)
 
     def get_retval(self) -> int:
         """Get return value."""
@@ -516,10 +551,6 @@ class Infernum:
         """Set return value."""
         self.uc.reg_write(self.arch.reg_ret, value)
 
-    def jump_back(self):
-        """Jump to the address of current function was called."""
-        self.uc.reg_write(self.arch.reg_pc, self.uc.reg_read(self.arch.reg_lr))
-
     def create_buffer(self, size: int) -> int:
         """Create a buffer with the size."""
         return self.memory_manager.alloc(size)
@@ -528,6 +559,7 @@ class Infernum:
         """Create a buffer that is initialized to the string."""
         address = self.memory_manager.alloc(len(string) + 1)
         self.write_string(address, string)
+
         return address
 
     def free(self, address: int):
@@ -595,10 +627,13 @@ class Infernum:
         """Call function with the symbol name."""
         symbol = self.find_symbol(symbol_name)
         address = symbol.address
+
         self._start_emulate(address, *args)
+
         return self.get_retval()
 
     def call_address(self, address, *args) -> int:
         """Call function at the address."""
         self._start_emulate(address, *args)
+
         return self.get_retval()
