@@ -2,7 +2,7 @@ import os
 import random
 import threading
 from functools import wraps
-from typing import TYPE_CHECKING, Any, Callable
+from typing import TYPE_CHECKING, Any, Callable, Optional
 
 from unicorn.unicorn import UC_HOOK_CODE_TYPE
 
@@ -11,37 +11,37 @@ if TYPE_CHECKING:
 
 
 def intercept(f: Callable[["Infernum"], Any]) -> UC_HOOK_CODE_TYPE:
+    """Intercept function call."""
+
     @wraps(f)
     def decorator(*args):
         emulator = args[-1]["emulator"]
-        f(emulator)
-        emulator.jump_back()
+        emulator.return_call(f(emulator))
+
+    return decorator
+
+
+def simply_return(retval: Optional[int] = None) -> UC_HOOK_CODE_TYPE:
+    """Simply return function call."""
+
+    @intercept
+    def decorator(_):
+        return retval
 
     return decorator
 
 
 @intercept
-def hook_ctype_get_mb_cur_max(emulator: "Infernum"):
-    """Intercept ``__ctype_get_mb_cur_max`` of ``libc.so``."""
-    emulator.set_retval(1)
-
-
-@intercept
-def hook_arc4random(emulator: "Infernum"):
+def hook_arc4random(_):
     """Intercept ``arc4random`` of ``libc.so``."""
-    emulator.set_retval(random.randint(0, 0x100000000))
-
-
-@intercept
-def hook_clock_nanosleep(emulator: "Infernum"):
-    """Intercept ``clock_nanosleep`` of ``libc.so``."""
-    emulator.set_retval(0)
+    return random.randint(0, 0x100000000)
 
 
 @intercept
 def hook_free(emulator: "Infernum"):
     """Intercept ``free`` of ``libc.so``."""
     addr = emulator.get_argument(0)
+
     emulator.memory_manager.free(addr)
 
 
@@ -50,12 +50,15 @@ def hook_getcwd(emulator: "Infernum"):
     """Intercept ``getcwd`` of ``libc.so``."""
     buf = emulator.get_argument(0)
     cwd = os.getcwd()
+
     if not buf:
         buf = emulator.create_string(cwd)
     else:
         emulator.write_string(buf, cwd)
+
     emulator.set_argument(0, buf)
-    emulator.set_retval(buf)
+
+    return buf
 
 
 @intercept
@@ -75,7 +78,8 @@ def hook_malloc(emulator: "Infernum"):
     """Intercept ``malloc`` of ``libc.so``."""
     size = emulator.get_argument(0)
     addr = emulator.memory_manager.alloc(size)
-    emulator.set_retval(addr)
+
+    return addr
 
 
 @intercept
@@ -84,8 +88,10 @@ def hook_memcpy(emulator: "Infernum"):
     dst = emulator.get_argument(0)
     src = emulator.get_argument(1)
     size = emulator.get_argument(2)
+
     emulator.write_bytes(dst, emulator.read_bytes(src, size))
-    emulator.set_retval(dst)
+
+    return dst
 
 
 @intercept
@@ -94,11 +100,7 @@ def hook_memset(emulator: "Infernum"):
     addr = emulator.get_argument(0)
     char = emulator.get_argument(1)
     size = emulator.get_argument(2)
+
     emulator.write_bytes(addr, bytes([char for _ in range(size)]))
-    emulator.set_retval(addr)
 
-
-@intercept
-def hook_nanosleep(emulator: "Infernum"):
-    """Intercept ``nanosleep`` of ``libc.so``."""
-    emulator.set_retval(0)
+    return addr
